@@ -2,23 +2,25 @@
   (:require [clojure.tools.logging :as log]
             [ring.util.response :as ring-response]
             [slingshot.slingshot :as sling]
-            [puppetlabs.ring-middleware.core :as mw]
+            [puppetlabs.ring-middleware.core :as middleware]
+            [puppetlabs.ring-middleware.utils :as ringutils]
+            [puppetlabs.services.protocols.jruby-puppet :as jruby-puppet]
             [puppetlabs.services.jruby.jruby-puppet-service :as jruby]
             [schema.core :as schema]
-            [puppetlabs.puppetserver.common :as ps-common]))
-
+            [puppetlabs.puppetserver.common :as ps-common]
+            [puppetlabs.i18n.core :as i18n :refer [trs tru]]))
 
 (defn jruby-timeout?
   "Determine if the supplied slingshot message is for a JRuby borrow timeout."
   [x]
   (when (map? x)
-    (= (:type x)
-       :puppetlabs.services.jruby.jruby-puppet-service/jruby-timeout)))
+    (= (:kind x)
+       :puppetlabs.services.jruby-pool-manager.jruby-core/jruby-timeout)))
 
 (defn output-error
-  [{:keys [uri]} {:keys [message]} http-status]
-  (log/errorf "Error %d on SERVER at %s: %s" http-status uri message)
-  (mw/plain-response http-status message))
+  [{:keys [uri]} {:keys [msg]} http-status]
+  (log/error (trs "Error {0} on SERVER at {1}: {2}" http-status uri msg))
+  (ringutils/plain-response http-status msg))
 
 (defn wrap-with-error-handling
   "Middleware that wraps a JRuby request with some error handling to return
@@ -27,11 +29,11 @@
   (fn [request]
     (sling/try+
      (handler request)
-     (catch mw/bad-request? e
+     (catch ringutils/bad-request? e
        (output-error request e 400))
      (catch jruby-timeout? e
        (output-error request e 503))
-     (catch mw/service-unavailable? e
+     (catch ringutils/service-unavailable? e
        (output-error request e 503)))))
 
 (defn wrap-with-jruby-instance
@@ -40,11 +42,8 @@
   [handler jruby-service]
   (fn [request]
     (jruby/with-jruby-puppet
-     jruby-instance
-     jruby-service
-     {:request (dissoc request :ssl-client-cert)}
-
-     (handler (assoc request :jruby-instance jruby-instance)))))
+     jruby-puppet jruby-service {:request (dissoc request :ssl-client-cert)}
+     (handler (assoc request :jruby-instance jruby-puppet)))))
 
 (defn get-environment-from-request
   "Gets the environment from the URL or query string of a request."
@@ -64,11 +63,11 @@
     (let [environment (get-environment-from-request request)]
       (cond
         (nil? environment)
-        (mw/throw-bad-request!
-         "An environment parameter must be specified")
+        (ringutils/throw-bad-request!
+         (tru "An environment parameter must be specified"))
 
         (not (nil? (schema/check ps-common/Environment environment)))
-        (mw/throw-bad-request!
+        (ringutils/throw-bad-request!
          (ps-common/environment-validation-error-msg environment))
 
         :else
