@@ -2,13 +2,16 @@
        "Core logic for the implementation of the PuppetServerConfigService in
         puppetlabs.services.config.puppet-server-config-service."}
     puppetlabs.services.config.puppet-server-config-core
-  (:import (com.puppetlabs.puppetserver JRubyPuppet))
   (:require [clojure.set :as set]
             [clojure.tools.logging :as log]
             [puppetlabs.kitchensink.core :refer [keyset]]
             [puppetlabs.services.jruby.jruby-puppet-service :as jruby]
+            [puppetlabs.services.jruby.jruby-puppet-service :as jruby]
             [schema.core :as schema]
-            [clojure.string :as str]))
+            [clojure.string :as str]
+            [puppetlabs.i18n.core :as i18n :refer [trs]]
+            [puppetlabs.services.protocols.jruby-puppet :as jruby-puppet])
+  (:import (com.puppetlabs.puppetserver JRubyPuppet)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; config
@@ -65,9 +68,8 @@
   (->> (keyword->setting k)
        (.getSetting jruby-puppet)))
 
-(defn get-puppet-config*
-  [jruby-puppet]
-  {:pre [(instance? JRubyPuppet jruby-puppet)]}
+(schema/defn get-puppet-config*
+  [jruby-puppet :- JRubyPuppet]
   (into {}
         (for [k puppet-config-keys]
           {k (get-puppet-config-value jruby-puppet k)})))
@@ -94,9 +96,7 @@
   (let [key-conflicts (set/intersection (keyset tk-config) puppet-config-keys)]
     (when-not (empty? key-conflicts)
       (throw (Exception.
-               (str "The following configuration keys "
-                    "conflict with the values to be "
-                    "read from puppet: "
+               (trs "The following configuration keys conflict with the values to be read from puppet: {0}"
                     (str/join ", " (sort key-conflicts))))))))
 
 (schema/defn ^:always-validate
@@ -105,9 +105,9 @@
   [jruby-service]
   {:post [(map? %)]}
   (jruby/with-jruby-puppet
-    jruby-puppet jruby-service :get-puppet-config
-    (let [config (get-puppet-config* jruby-puppet)]
-      (assoc config :puppet-version (.puppetVersion jruby-puppet)))))
+   jruby-puppet jruby-service :get-puppet-config
+   (let [config (get-puppet-config* jruby-puppet)]
+     (assoc config :puppet-version (.puppetVersion jruby-puppet)))))
 
 (defn init-webserver!
   "Initialize Jetty with paths to the master's SSL certs."
@@ -118,8 +118,7 @@
                    :ssl-ca-cert  localcacert
                    :ssl-crl-path cacrl}]
     (if (some #((key %) webserver-settings) overrides)
-      (log/info
-        "Not overriding webserver settings with values from core Puppet")
+      (log/info (trs "Not overriding webserver settings with values from core Puppet"))
       (do
-        (log/info "Initializing webserver settings from core Puppet")
+        (log/info (trs "Initializing webserver settings from core Puppet"))
         (override-webserver-settings! overrides)))))
