@@ -1,6 +1,7 @@
 (ns puppetlabs.services.jruby.puppet-environments
   (:import (com.puppetlabs.puppetserver EnvironmentRegistry))
-  (:require [clojure.tools.logging :as log]))
+  (:require [clojure.tools.logging :as log]
+            [puppetlabs.i18n.core :as i18n :refer [trs]]))
 
 (defprotocol EnvironmentStateContainer
   (environment-state [this])
@@ -18,26 +19,26 @@
       EnvironmentRegistry
       (registerEnvironment [this env-name]
         (when-not env-name
-          (throw (IllegalArgumentException. "Missing environment name!")))
+          (throw (IllegalArgumentException. (trs "Missing environment name!"))))
         (log/debugf "Registering environment '%s'" env-name)
         (swap! state assoc-in [(keyword env-name) :expired] false)
         nil)
       (isExpired [this env-name]
         (when-not env-name
-          (throw (IllegalArgumentException. "Missing environment name!")))
+          (throw (IllegalArgumentException. (trs "Missing environment name!"))))
         (get-in @state [(keyword env-name) :expired]))
       (removeEnvironment [this env-name]
         (when-not env-name
-          (throw (IllegalArgumentException. "Missing environment name!")))
-        (log/debugf "Removing environment '%s' from registry" env-name)
+          (throw (IllegalArgumentException. (trs "Missing environment name!"))))
+        (log/debug (trs "Removing environment ''{0}'' from registry" env-name))
         (swap! state dissoc (keyword env-name))
         nil)
 
       EnvironmentStateContainer
       (environment-state [this] state)
       (mark-all-environments-expired! [this]
-        (log/info "Marking all registered environments as expired.")
+        (log/info (trs "Marking all registered environments as expired."))
         (swap! state mark-all-expired!))
       (mark-environment-expired! [this env-name]
-        (log/infof "Marking environment '%s' as expired." env-name)
+        (log/info (trs "Marking environment ''{0}'' as expired." env-name))
         (swap! state mark-expired! (keyword env-name))))))
