@@ -14,10 +14,12 @@
             [puppetlabs.puppetserver.jruby-request :as jruby-request]
             [puppetlabs.kitchensink.core :as ks]
             [puppetlabs.ring-middleware.core :as middleware]
+            [puppetlabs.ring-middleware.utils :as middleware-utils]
             [cheshire.core :as cheshire]
             [clojure.string :as str]
             [bidi.schema :as bidi-schema]
-            [ring.middleware.params :as ring]))
+            [ring.middleware.params :as ring]
+            [puppetlabs.i18n.core :as i18n :refer [trs tru]]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Constants
@@ -138,7 +140,7 @@
   (if-let [obj-as-keyword (-> obj obj-or-ruby-symbol-as-string keyword)]
     obj-as-keyword
     (throw (IllegalArgumentException.
-            (str "Object cannot be coerced to a keyword: " obj)))))
+            (trs "Object cannot be coerced to a keyword: {0}" obj)))))
 
 (defn sort-nested-environment-class-info-maps
   "For a data structure, recursively sort any nested maps and sets descending
@@ -297,7 +299,7 @@
           (not-modified-response parsed-tag)
           (-> (response-with-etag info-as-json parsed-tag)
               (rr/content-type "application/json"))))
-      (middleware/json-response 200 info-for-json))))
+      (middleware-utils/json-response 200 info-for-json))))
 
 (schema/defn ^:always-validate
   environment-class-info-fn :- IFn
@@ -322,7 +324,7 @@
                                      (if-none-match-from-request request)
                                      cache-generation-id
                                      environment-class-cache-enabled)
-        (rr/not-found (str "Could not find environment '" environment "'"))))))
+        (rr/not-found (tru "Could not find environment ''{0}''" environment))))))
 
 (schema/defn ^:always-validate
   wrap-with-etag-check :- IFn
@@ -380,7 +382,7 @@
       (cond
         (some empty? [environment code-id file-path])
         {:status 400
-         :body "Error: A /static_file_content request requires an environment, a code-id, and a file-path"}
+         :body (tru "Error: A /static_file_content request requires an environment, a code-id, and a file-path")}
         (not (nil? (schema/check ps-common/Environment environment)))
         {:status 400
          :body (ps-common/environment-validation-error-msg environment)}
@@ -391,8 +393,7 @@
 
         (not (valid-static-file-path? file-path))
         {:status 403
-         :body (str "Request Denied: A /static_file_content request must be a file within "
-         "the files directory of a module.")}
+         :body (tru "Request Denied: A /static_file_content request must be a file within the files directory of a module.")}
 
         :else
         {:status 200
@@ -505,17 +506,10 @@
           required-mem-size (* heap-size 1.1)]
       (when (< mem-size required-mem-size)
         (throw (Error.
-                 (str "Not enough available RAM (" (int (/ mem-size 1024.0))
-                      "MB) to safely accommodate the configured JVM heap "
-                      "size of " (int (/ heap-size 1024.0)) "MB.  "
-                      "Puppet Server requires at least "
-                      (int (/ required-mem-size 1024.0))
-                      "MB of available RAM given this heap size, computed as "
-                      "1.1 * max heap (-Xmx).  Either increase available "
-                      "memory or decrease the configured heap size by "
-                      "reducing the -Xms and -Xmx values in JAVA_ARGS in "
-                      "/etc/sysconfig/puppetserver on EL systems or "
-                      "/etc/default/puppetserver on Debian systems.")))))))
+                (format "%s  %s  %s"
+                        (trs "Not enough available RAM ({0}MB) to safely accommodate the configured JVM heap size of {1}MB." (int (/ mem-size 1024.0)))
+                        (trs "Puppet Server requires at least {2}MB of available RAM given this heap size, computed as 1.1 * max heap (-Xmx)." (int (/ mem-size 1024.0)))
+                        (trs "Either increase available memory or decrease the configured heap size by reducing the -Xms and -Xmx values in JAVA_ARGS in /etc/sysconfig/puppetserver on EL systems or /etc/default/puppetserver on Debian systems." (int (/ mem-size 1024.0))))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
@@ -542,6 +536,7 @@
   (-> handler
       (middleware/wrap-uncaught-errors :plain)
       middleware/wrap-request-logging
+      i18n/locale-negotiator
       middleware/wrap-response-logging
       (ringutils/wrap-with-puppet-version-header puppet-version)))
 
@@ -573,7 +568,7 @@
 
     :else
     (throw (IllegalArgumentException.
-             (str "Route not found for service " master-ns)))))
+             (trs "Route not found for service {0}" master-ns)))))
 
 (schema/defn ^:always-validate
   get-wrapped-handler :- IFn
