@@ -1,16 +1,34 @@
 (ns puppetlabs.trapperkeeper.internal
-  (:import (clojure.lang ExceptionInfo IFn IDeref))
+  (:import (clojure.lang ExceptionInfo IFn IDeref)
+           (java.lang ArithmeticException NumberFormatException)
+           (java.io FileNotFoundException))
   (:require [clojure.tools.logging :as log]
             [beckon]
             [plumbing.graph :as graph]
+            [slingshot.slingshot :refer [throw+]]
             [puppetlabs.kitchensink.core :refer [add-shutdown-hook! boolean? cli!]]
-            [puppetlabs.trapperkeeper.config :refer [config-service]]
+            [puppetlabs.trapperkeeper.config :refer [config-service get-in-config]]
             [puppetlabs.trapperkeeper.app :as a]
+            [puppetlabs.trapperkeeper.common :as common]
             [puppetlabs.trapperkeeper.services :as s]
             [puppetlabs.kitchensink.core :as ks]
             [schema.core :as schema]
             [clojure.core.async :as async]
-            [clojure.core.async.impl.protocols :as async-prot]))
+            [clojure.core.async.impl.protocols :as async-prot]
+            [clojure.string :as string]
+            [me.raynes.fs :as fs]))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Schemas
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(def LifeCycleTask
+  {:type (schema/enum :restart :shutdown :boot)
+   :task-function IFn})
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Private
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;; This is (eww) a global variable that holds a reference to all of the running
 ;; Trapperkeeper apps in the process. It can be used when connecting via nrepl
@@ -18,31 +36,54 @@
 ;; (such as signal handling).
 (def tk-apps (atom []))
 
+(def max-pending-lifecycle-events 5)
+
 (defn service-graph?
   "Predicate that tests whether or not the argument is a valid trapperkeeper
   service graph."
   [service-graph]
   (and
-    (map? service-graph)
-    (every? keyword? (keys service-graph))
-    (every? (some-fn ifn? service-graph?) (vals service-graph))))
+   (map? service-graph)
+   (every? keyword? (keys service-graph))
+   (every? (some-fn ifn? service-graph?) (vals service-graph))))
+
+(defn inc-restart-counter!
+  "Increments the counter in a restart-file for purposes of supporting HUP behavior"
+  [app]
+  (when-let
+      [restart-file-path (get-in-config (a/get-service app :ConfigService) [:global :restart-file])]
+      (try
+        (if (fs/exists? restart-file-path)
+          (if (and (fs/writeable? restart-file-path) (fs/readable? restart-file-path))
+            (spit restart-file-path (inc (Long/parseLong (slurp restart-file-path))))
+            (throw (IllegalStateException. (format "Restart file %s is not readable and/or writeable" restart-file-path))))
+          (let [dir-path (fs/parent restart-file-path)]
+            (fs/mkdirs dir-path)
+            (spit restart-file-path "1")))
+        (catch ArithmeticException e
+          (spit restart-file-path "1")
+          (log/debug "Number of restarts has exceeded Long/MAX_VALUE, resetting file to 1"))
+        (catch NumberFormatException e
+          (spit restart-file-path "1")
+          (log/error "Restart file is unparseable, resetting file to 1")))))
 
 (defn validate-service-graph!
   "Validates that a ServiceDefinition contains a valid trapperkeeper service graph.
-  Returns the service definition on success; throws an IllegalArgumentException
+  Returns the service definition on success; throws an ::invalid-service-graph
   if the graph is invalid."
   [service-def]
   {:post [(satisfies? s/ServiceDefinition %)]}
   (if-not (satisfies? s/ServiceDefinition service-def)
-    (throw (IllegalArgumentException.
-             (str "Invalid service definition; expected a service "
-                  "definition (created via `service` or `defservice`); "
-                  "found: " (pr-str service-def)))))
+    (throw+ {:type ::invalid-service-graph
+             :message (str "Invalid service definition; expected a service "
+                           "definition (created via `service` or `defservice`); "
+                           "found: " (pr-str service-def))}))
   (if (service-graph? (s/service-map service-def))
     service-def
-    (throw (IllegalArgumentException. (str "Invalid service graph; service graphs must "
-                                           "be nested maps of keywords to functions.  Found: "
-                                           (s/service-map service-def))))))
+    (throw+ {:type ::invalid-service-graph
+             :message (str "Invalid service graph; service graphs must "
+                           "be nested maps of keywords to functions.  Found: "
+                           (s/service-map service-def))})))
 
 (defn parse-missing-required-key
   "Prismatic's graph compilation code throws `ExceptionInfo` objects if required
@@ -88,13 +129,13 @@
 
       (if (sequential? (:error data))
         (let [missing-services (keys (ks/filter-map
-                                       (fn [_ v] (= v 'missing-required-key))
-                                       (.error (first (:error data)))))]
+                                      (fn [_ v] (= v 'missing-required-key))
+                                      (.error (first (:error data)))))]
           (if (= 1 (count missing-services))
             (throw (RuntimeException.
-                     (format "Service '%s' not found" (first missing-services))))
+                    (format "Service '%s' not found" (first missing-services))))
             (throw (RuntimeException.
-                     (format "Services '%s' not found" missing-services)))))
+                    (format "Services '%s' not found" missing-services)))))
         (throw e)))))
 
 (defn compile-graph
@@ -121,16 +162,14 @@
       (handle-prismatic-exception! e))))
 
 
-(defn parse-cli-args!
+(schema/defn parse-cli-args! :- common/CLIData
   "Parses the command-line arguments using `puppetlabs.kitchensink.core/cli!`.
   Hard-codes the command-line arguments expected by trapperkeeper to be:
       --debug
       --bootstrap-config <bootstrap file>
       --config <.ini file or directory>
       --plugins <plugins directory>"
-  [cli-args]
-  {:pre  [(sequential? cli-args)]
-   :post [(map? %)]}
+  [cli-args :- [(schema/maybe schema/Str)]]
   (let [specs       [["-d" "--debug" "Turns on debug mode"]
                      ["-b" "--bootstrap-config BOOTSTRAP-CONFIG-FILE" "Path to bootstrap config file"]
                      ["-c" "--config CONFIG-PATH"
@@ -161,11 +200,11 @@
         updated-ctxt  (lifecycle-fn s (get-in @app-context [:service-contexts service-id] {}))]
     (if-not (map? updated-ctxt)
       (throw (IllegalStateException.
-               (format
-                 "Lifecycle function '%s' for service '%s' must return a context map (got: %s)"
-                 lifecycle-fn-name
-                 (or (s/service-symbol s) service-id)
-                 (pr-str updated-ctxt)))))
+              (format
+               "Lifecycle function '%s' for service '%s' must return a context map (got: %s)"
+               lifecycle-fn-name
+               (or (s/service-symbol s) service-id)
+               (pr-str updated-ctxt)))))
     ;; store the updated service context map in the application context atom
     (swap! app-context assoc-in [:service-contexts service-id] updated-ctxt)))
 
@@ -205,24 +244,60 @@
   "Initializes a 'worker' which will listen for lifecycle-related tasks and perform
   them on a background thread, to ensure that we aren't executing multiple lifecycle
   tasks simultaneously."
-  [lifecycle-chan :- (schema/protocol async-prot/Channel)
+  [lifecycle-channel :- (schema/protocol async-prot/Channel)
+   shutdown-channel :- (schema/protocol async-prot/Channel)
    shutdown-reason-promise :- IDeref]
-  (async/go (try
-              (loop []
-                (let [task (async/<! lifecycle-chan)]
-                  (if (= task :shutdown)
-                    (do
-                      (log/debug "Received shutdown command, lifecycle worker exiting.")
-                      :done)
-                    (do
-                      (log/debug "Lifecycle worker executing lifecycle task.")
-                      (task)
-                      (log/debug "Lifecycle worker completed lifecycle task; awaiting next task.")
-                      (recur)))))
+  (log/debug "Initializing lifecycle worker loop.")
+  (async/go-loop []
+    (let [[task chan] (async/alts!
+                       [shutdown-channel lifecycle-channel]
+                       :priority true)]
+      (schema/validate LifeCycleTask task)
+      (let [{:keys [type task-function]} task]
+        (condp #(contains? %1 %2) type
+          #{:shutdown}
+          (do
+            (try
+              (log/debug "Received shutdown command, shutting down services")
+              (async/close! shutdown-channel)
+              (async/close! lifecycle-channel)
+
+              (log/debug "Clearing lifecycle worker channels for shutdown.")
+              ;; drain the channels to ensure that there are
+              ;; no blocking puts, e.g. if a second shutdown request
+              ;; was queued simultaneously
+              (ks/while-let [msg (async/poll! shutdown-channel)]
+                            (log/debug "Shutdown in progress, ignoring message on shutdown channel:"
+                                       (:type msg)))
+              (ks/while-let [msg (async/poll! lifecycle-channel)]
+                            (log/debug "Shutdown in progress, ignoring message on main lifecycle channel:"
+                                       (:type msg)))
+
+              (task-function)
+
+              (log/debug "Service shutdown complete, exiting lifecycle worker loop")
               (catch Exception e
+                (log/debug e "Exception caught during shutdown!")))
+            :done)
+
+          #{:boot :restart}
+          (do
+            (try
+              (log/debugf "Lifecycle worker executing %s lifecycle task." type)
+              (task-function)
+              (log/debugf "Lifecycle worker completed %s lifecycle task; awaiting next task." type)
+              (catch Exception e
+                (log/debug e "Exception caught in lifecycle worker loop")
                 (deliver shutdown-reason-promise
                          {:cause :service-error
-                          :error e})))))
+                          :error e})))
+            (recur))
+
+          (do
+            (deliver shutdown-reason-promise
+                     {:cause :service-error
+                      :error (IllegalStateException. "Unrecognized lifecycle task:" task)})
+            (recur)))))))
 
 (defn restart-tk-apps
   "Call restart on all tk apps."
@@ -231,7 +306,11 @@
   (doseq [app apps]
     (let [{:keys [lifecycle-channel]} @(a/app-context app)
           restart-fn #(a/restart app)]
-      (async/>!! lifecycle-channel restart-fn))))
+      (when-not (async/offer! lifecycle-channel
+                              {:type :restart
+                               :task-function restart-fn})
+        (log/warnf "Ignoring new SIGHUP restart requests; too many requests queued (%s)"
+                   max-pending-lifecycle-events)))))
 
 (defn register-sighup-handler
   "Register a handler for SIGHUP that restarts all trapperkeeper apps. The
@@ -344,7 +423,7 @@
   (s/service ShutdownService
     []
     (get-shutdown-reason [this] (when (realized? shutdown-reason-promise)
-                                      @shutdown-reason-promise))
+                                  @shutdown-reason-promise))
     (wait-for-shutdown [this] (deref shutdown-reason-promise))
     (request-shutdown [this]  (request-shutdown* shutdown-reason-promise))
     (shutdown-on-error [this svc-id f] (shutdown-on-error* shutdown-reason-promise app-context svc-id f))
@@ -355,19 +434,24 @@
    in reverse order (to account for dependency relationships)."
   [app-context :- (schema/atom a/TrapperkeeperAppContext)]
   (log/info "Beginning shutdown sequence")
-  (let [{:keys [ordered-services lifecycle-channel lifecycle-worker]} @app-context]
-    (doseq [[service-id s] (reverse ordered-services)]
-      (try
-        (run-lifecycle-fn! app-context s/stop "stop" service-id s)
-        (catch Exception e
-          (log/error e "Encountered error during shutdown sequence"))))
-    (when-not (async-prot/closed? lifecycle-worker)
-      (log/debug "Service shutdown complete, shutting down lifecycle worker")
-      (async/>!! lifecycle-channel :shutdown)
-      ;; wait for the channel to send us the return value so we know it's done
-      (async/<!! lifecycle-worker)
-      (log/debug "Lifecycle worker shutdown complete"))
-    (log/info "Finished shutdown sequence")))
+  (let [{:keys [ordered-services shutdown-channel lifecycle-worker]} @app-context
+        shutdown-fn (fn [] (doseq [[service-id s] (reverse ordered-services)]
+                             (try
+                               (run-lifecycle-fn! app-context s/stop "stop" service-id s)
+                               (catch Exception e
+                                 (log/error e "Encountered error during shutdown sequence")))))]
+    (log/trace "Putting shutdown message on shutdown channel.")
+    (async/>!! shutdown-channel {:type :shutdown
+                                 :task-function shutdown-fn})
+    ;; wait for the channel to send us the return value so we know it's done
+    (log/trace "Waiting for response to shutdown message from lifecycle worker.")
+    (if (not (nil? (async/<!! lifecycle-worker)))
+      (log/info "Finished shutdown sequence")
+      ;; else, the read from the channel returned a nil because it was closed,
+      ;; indicating that there was already a shutdown in progress, and thus the
+      ;; redundant shutdown request was ignored
+      (log/trace (str "Response from lifecycle worker indicates shutdown already in progress,"
+                      "ignoring additional shutdown attempt.")))))
 
 (schema/defn ^:always-validate initialize-shutdown-service! :- (schema/protocol s/ServiceDefinition)
   "Initialize the shutdown service and add a shutdown hook to the JVM."
@@ -452,7 +536,8 @@
   [services :- [(schema/protocol s/ServiceDefinition)]
    config-data-fn :- IFn]
   (let [shutdown-reason-promise (promise)
-        lifecycle-channel (async/chan 5)
+        lifecycle-channel (async/chan max-pending-lifecycle-events)
+        shutdown-channel (async/chan)
         ;; this is the application context for this app instance.  its keys
         ;; will be the service ids, and values will be maps that represent the
         ;; context for each individual service
@@ -460,8 +545,10 @@
                            :ordered-services []
                            :services-by-id {}
                            :lifecycle-channel lifecycle-channel
+                           :shutdown-channel shutdown-channel
                            :lifecycle-worker (initialize-lifecycle-worker
                                               lifecycle-channel
+                                              shutdown-channel
                                               shutdown-reason-promise)
                            :shutdown-reason-promise shutdown-reason-promise})
         service-refs (atom {})
@@ -492,12 +579,13 @@
       (a/service-graph [this] graph-instance)
       (a/app-context [this] app-context)
       (a/check-for-errors! [this] (throw-app-error-if-exists!
-                                    this))
+                                   this))
       (a/init [this]
         (run-lifecycle-fns app-context s/init "init" ordered-services)
         this)
       (a/start [this]
         (run-lifecycle-fns app-context s/start "start" ordered-services)
+        (inc-restart-counter! this)
         this)
       (a/stop [this]
         (shutdown! app-context)
@@ -508,6 +596,7 @@
           (doseq [svc-id (keys services-by-id)] (swap! app-context assoc-in [:service-contexts svc-id] {}))
           (run-lifecycle-fns app-context s/init "init" ordered-services)
           (run-lifecycle-fns app-context s/start "start" ordered-services)
+          (inc-restart-counter! this)
           this
           (catch Throwable t
             (deliver shutdown-reason-promise {:cause :service-error
@@ -532,7 +621,8 @@
   (let [lifecycle-channel (:lifecycle-channel @(a/app-context app))
         lifecycle-promise (promise)
         boot-fn (partial boot-services-for-app** lifecycle-promise app)]
-    (async/>!! lifecycle-channel boot-fn)
+    (async/>!! lifecycle-channel {:type :boot
+                                  :task-function boot-fn})
     @lifecycle-promise
     app))
 
